"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var moment = require("moment");
var rp = require("request-promise-native");
var NodeSSH = require('node-ssh').NodeSSH;
var log4js = require("log4js");
var config_1 = require("../config/config");
var rasa_config_1 = require("../config/rasa.config");
var mssql_service_1 = require("../db/mssql.service");
var redis_service_1 = require("../db/redis.service");
var promises_shared_1 = require("../shared/promises.shared");
var file_manager_1 = require("../shared/file-manager");
var utilities_1 = require("../shared/utilities");
var platform_service_1 = require("../services/platform.service");
var logger = log4js.getLogger('Task');
var RASAServiceImplementation = (function () {
    function RASAServiceImplementation() {
        this.mySsh = new NodeSSH();
    }
    RASAServiceImplementation.prototype.GetCognitiveEngineName = function () {
        return 'RASA';
    };
    RASAServiceImplementation.prototype.GetCognitiveEngineCharsPerPhrase = function () {
        return config_1.COGNITIVE_SERVICES_RASA_CHARS_PER_PHRASE;
    };
    RASAServiceImplementation.prototype.AddApp = function (subscriptionRegion, subscriptionKey, nlu) {
        var _this = this;
        var installationId = '';
        var vcc = nlu.vcc;
        var nluName = nlu.name;
        var appId = '';
        var rootFolder = config_1.RASA_ROOT_FOLDER;
        var nluLocalFolder = config_1.PATH_FOLDER_TEMP_NLU + '/' + vcc;
        var nluRemoteFolder = null;
        var ssh = null;
        return promises_shared_1.DummyPromise()
            .then(function (result) {
            return platform_service_1.GetPlatformVersion();
        }).then(function (result) {
            logger.info('[RASAServiceImplementation::AddApp] Getting result:', result, 'after GetVersion');
            if (result) {
                installationId = result.installationId;
                return _this.GetSsh();
            }
            else {
                throw 'SERVER_ERROR_NO_VERSION_DATA';
            }
        }).then(function (result) {
            if (!result)
                throw 'SERVER_ERROR_CANNOT_CONNECT_RASA_SERVER';
            ssh = result;
            var cmd = 'ls -d *' + installationId + '::*';
            return ssh.execCommand(cmd, { cwd: rootFolder });
        }).then(function (result) {
            if (result === null)
                throw 'SERVER_ERROR_NO_RESULT_FROM_RASA_SERVER';
            var arrContent = result.stdout.split(/\r?\n/);
            if (arrContent.includes(vcc + '::' + nlu.name))
                throw 'MODEL_NLU_ALREADY_EXIST';
            var cmd = 'mkdir -p ';
            appId = installationId + '::' + vcc + '::' + nluName.toLocaleLowerCase();
            cmd += appId;
            return ssh.execCommand(cmd, { cwd: rootFolder });
        }).then(function (result) {
            logger.debug('[RASAServiceImplementation::AddApp] Create nlu folder: ' + JSON.stringify(result));
            return file_manager_1.CreateFolder(nluLocalFolder);
        }).then(function (result) {
            logger.debug('[RASAServiceImplementation::AddApp] Create folder: ' + JSON.stringify(result));
            var regex = new RegExp("(?<=^language:\\s).*$", 'm');
            var nluConfigContent = rasa_config_1.RASA_NLU_CONFIG_CONTENT;
            var newNluConfigTextContent = nluConfigContent.replace(regex, nlu.culture);
            var tempModelConfigPath = nluLocalFolder + '/' + nluName + '-config.yml';
            var promiseArray = [];
            promiseArray.push(file_manager_1.WriteFileContent(tempModelConfigPath, newNluConfigTextContent, 'utf-8'));
            nluRemoteFolder = rootFolder + '/' + appId;
            promiseArray.push(ssh.execCommand('mkdir -p data', { cwd: nluRemoteFolder }));
            promiseArray.push(ssh.execCommand('mkdir -p models', { cwd: nluRemoteFolder }));
            return Promise.all(promiseArray);
        }).then(function (result) {
            logger.debug('[RASAServiceImplementation::AddApp] Create nlu: ' + JSON.stringify(result));
            if (nlu.importNLU) {
                switch (nlu.importNLUFrom) {
                    case 'RASA': return _this.ImportNLUFromRASA(nlu);
                    default: throw 'SERVER_ERROR_NOT_FOUND_COGNITIVE_ENGINE_SOURCE';
                }
            }
            else {
                return Promise.resolve(true);
            }
        }).then(function (result) {
            var promiseArray = [];
            promiseArray.push(ssh.putFile(nluLocalFolder + '/' + nluName + '-config.yml', nluRemoteFolder + '/config.yml'));
            promiseArray.push(ssh.putFile(config_1.STATIC_PY_FOLDER_PATH + '/nlu-train-data-generator.py', nluRemoteFolder + '/trainDataGenerator.py'));
            if (nlu.importNLU) {
                promiseArray.push(ssh.putFile(nluLocalFolder + '/' + nluName + '-nlu.md', nluRemoteFolder + '/data/nlu.md'));
            }
            else {
                promiseArray.push(ssh.execCommand('touch nlu.md', { cwd: (nluRemoteFolder + '/data') }));
            }
            return Promise.all(promiseArray);
        }).then(function (result) {
            var cmd = 'ls ';
            var modelsFolder = nluRemoteFolder + '/models';
            return ssh.execCommand(cmd, { cwd: modelsFolder });
        }).then(function (result) {
            logger.debug('[RASAServiceImplementation::AddApp] Then: ' + JSON.stringify(result));
            return Promise.resolve(vcc + '::' + nluName.toLowerCase());
        }).catch(function (error) {
            var errorMessage = typeof error === 'string' ? error : 'ERROR';
            logger.error('[RASAServiceImplementation::AddApp] Error: ', error);
            if (error.code && error.code === 'ETIMEDOUT')
                errorMessage = 'SERVER_ERROR_TIMEOUT_FROM_RASA_SERVER';
            if (error.code && error.code === 'ECONNREFUSED')
                errorMessage = 'SERVER_ERROR_CONNECTION_REFUSED_FROM_RASA_SERVER';
            return Promise.reject(errorMessage);
        });
    };
    RASAServiceImplementation.prototype.DeleteApp = function (subscriptionRegion, subscriptionKey, nlu) {
        var _this = this;
        var installationId = '';
        var vcc = nlu.vcc;
        var nluName = nlu.name;
        var appId = '';
        var rootFolder = config_1.RASA_ROOT_FOLDER;
        var nluFolder = null;
        var ssh = null;
        return promises_shared_1.DummyPromise()
            .then(function (result) {
            return platform_service_1.GetPlatformVersion();
        }).then(function (result) {
            logger.info('[RASAServiceImplementation::AddApp] Getting result:', result, 'after GetVersion');
            if (result) {
                installationId = result.installationId;
                return _this.GetSsh();
            }
            else {
                throw 'SERVER_ERROR_NO_VERSION_DATA';
            }
        }).then(function (result) {
            if (!result)
                throw 'SERVER_ERROR_CANNOT_CONNECT_RASA_SERVER';
            ssh = result;
            var cmd = 'ls -d *' + installationId + '::*';
            return ssh.execCommand(cmd, { cwd: rootFolder });
        }).then(function (result) {
            if (result === null)
                throw 'SERVER_ERROR_NO_RESULT_FROM_RASA_SERVER';
            var arrContent = result.stdout.split(/\r?\n/);
            appId = installationId + '::' + vcc + '::' + nluName.toLowerCase();
            if (!arrContent.includes(appId))
                throw 'MODEL_NLU_NOT_FOUND';
            var cmd = 'rm -rf ';
            cmd += appId;
            return ssh.execCommand(cmd, { cwd: rootFolder });
        }).then(function (result) {
            var cmd = 'ls -d *' + installationId + '::*';
            return ssh.execCommand(cmd, { cwd: rootFolder });
        }).then(function (result) {
            if (result === null)
                throw 'SERVER_ERROR_NO_RESULT_FROM_RASA_SERVER';
            var arrContent = result.stdout.split(/\r?\n/);
            appId = installationId + '::' + vcc + '::' + nluName.toLowerCase();
            if (!arrContent.includes(appId))
                return Promise.resolve(true);
            else
                throw 'SERVER_ERROR_COULDNOT_DELETE_MODEL';
        }).catch(function (error) {
            var errorMessage = typeof error === 'string' ? error : 'ERROR';
            logger.error('[RASAServiceImplementation::DeleteApp] Error: ', error);
            if (error.code && error.code === 'ETIMEDOUT')
                errorMessage = 'SERVER_ERROR_TIMEOUT_FROM_RASA_SERVER';
            if (error.code && error.code === 'ECONNREFUSED')
                errorMessage = 'SERVER_ERROR_CONNECTION_REFUSED_FROM_RASA_SERVER';
            return Promise.reject(errorMessage);
        });
    };
    RASAServiceImplementation.prototype.GetAppVersion = function (subscriptionRegion, subscriptionKey, appId) {
        var installationId = '';
        var vcc = appId.split('::')[0];
        var model = appId.split('::')[1];
        return promises_shared_1.DummyPromise()
            .then(function (result) {
            return mssql_service_1.MSSql.GetNLU(vcc, model);
        }).then(function (result) {
            return Promise.resolve(result);
        }).catch(function (error) {
            var errorMessage = typeof error === 'string' ? error : 'ERROR';
            logger.error('[RASAServiceImplementation::GetAppVersion] Error: ', error);
            return Promise.reject(errorMessage);
        });
    };
    RASAServiceImplementation.prototype.GetAppsList = function (subscriptionRegion, subscriptionKey) {
        var _this = this;
        var installationId = '';
        var rootFolder = config_1.RASA_ROOT_FOLDER;
        var ssh = null;
        return promises_shared_1.DummyPromise()
            .then(function (result) {
            return platform_service_1.GetPlatformVersion();
        }).then(function (result) {
            logger.info('[RASAServiceImplementation::GetAppsList] Getting result:', result, 'after GetVersion');
            if (result) {
                installationId = result.installationId;
                return _this.GetSsh();
            }
            else {
                throw 'ERROR_NO_VERSION_DATA';
            }
        }).then(function (result) {
            ssh = result;
            var cmd = 'ls -d *' + installationId + '::*';
            return ssh.execCommand(cmd, { cwd: rootFolder });
        }).then(function (result) {
            if (result === null)
                throw 'SERVER_ERROR_NO_RESULT_FROM_RASA_SERVER';
            var arrContent = result.stdout.split(/\r?\n/);
            var res = [];
            arrContent.map(function (i) { return res.push({ id: i.split('::')[1] + '::' + i.split('::')[2] }); });
            return Promise.resolve(res);
        }).catch(function (error) {
            var errorMessage = typeof error === 'string' ? error : 'ERROR';
            logger.error('[RASAServiceImplementation::GetAppsList] Error: ', error);
            if (error.code && error.code === 'ETIMEDOUT')
                errorMessage = 'SERVER_ERROR_TIMEOUT_FROM_RASA_SERVER';
            if (error.code && error.code === 'ECONNREFUSED')
                errorMessage = 'SERVER_ERROR_CONNECTION_REFUSED_FROM_RASA_SERVER';
            return Promise.reject(errorMessage);
        });
    };
    RASAServiceImplementation.prototype.GetCultures = function (subscriptionRegion, subscriptionKey) {
        var cultures = [];
        cultures.push({ name: "Chinese", code: "zh" });
        cultures.push({ name: "Dutch", code: "nl" });
        cultures.push({ name: "English", code: "en" });
        cultures.push({ name: "French", code: "fr" });
        cultures.push({ name: "German", code: "de" });
        cultures.push({ name: "Italian", code: "it" });
        cultures.push({ name: "Portuguese", code: "pt" });
        cultures.push({ name: "Spanish", code: "es" });
        return Promise.resolve(cultures.filter(function (c) { return config_1.COGNITIVE_SERVICES_RASA_AVAILABLE_LANGUAGES.includes(c.code); }));
    };
    RASAServiceImplementation.prototype.AddIntent = function (subscriptionRegion, subscriptionKey, appId, name) {
        var _this = this;
        var installationId = '';
        var vcc = appId.split('::')[0];
        var model = appId.split('::')[1];
        var rootFolder = config_1.RASA_ROOT_FOLDER;
        var modelFolder = '';
        var nluLocalFolder = config_1.PATH_FOLDER_TEMP_NLU + '/' + vcc;
        var oldNluTextContent = '';
        var newNluTextContent = '';
        var ssh = null;
        return promises_shared_1.DummyPromise()
            .then(function (result) {
            if (name.toLowerCase() === 'none')
                throw 'SERVER_ERROR_INTENT_ID_INVALID';
            return platform_service_1.GetPlatformVersion();
        }).then(function (result) {
            logger.info('[RASAServiceImplementation::AddIntent] Getting result:', result, 'after GetVersion');
            if (result) {
                installationId = result.installationId;
                return _this.GetSsh();
            }
            else {
                throw 'ERROR_NO_VERSION_DATA';
            }
        }).then(function (result) {
            ssh = result;
            var cmd = 'cat nlu.md';
            modelFolder = rootFolder + '/' + installationId + '::' + appId.toLowerCase();
            return ssh.execCommand(cmd, { cwd: modelFolder + '/data' });
        }).then(function (result) {
            if (result === null)
                throw 'SERVER_ERROR_NO_RESULT_FROM_RASA_SERVER';
            oldNluTextContent = result.stdout;
            if (!oldNluTextContent.includes('## intent:None'))
                oldNluTextContent += '\n\n## intent:None\n- xxxxx\n\n';
            var regex = new RegExp("#{2}\\sintent:" + name + "\\s*$\n", 'm');
            var searchOnContent = oldNluTextContent.match(regex);
            if (searchOnContent)
                throw 'SERVER_ERROR_INTENT_ALREADY_EXIST';
            newNluTextContent = "\n## intent:" + name + "\n\n" + oldNluTextContent;
            return file_manager_1.CreateFolder(nluLocalFolder);
        }).then(function (result) {
            if (!result)
                throw 'SERVER_ERROR_CANNOT_CREATE_NLU_TEMPORARY_FOLDER';
            var promiseArray = [];
            var tempModelPath = nluLocalFolder + '/' + model + '.md';
            promiseArray.push(file_manager_1.WriteFileContent(tempModelPath, newNluTextContent, 'utf-8'));
            var cmd = 'cp -R nlu.md nlu-backup.md';
            promiseArray.push(ssh.execCommand(cmd, { cwd: modelFolder + '/data' }));
            return Promise.all(promiseArray);
        }).then(function (result) {
            if (!result)
                throw 'SERVER_ERROR_CREATE_INTENT_FAILED';
            if (result && !result[0])
                throw 'SERVER_ERROR_CANNOT_CREATE_NLU_TEMPORARY_FILE';
            if (result && result[1] && result[1].stderr !== '')
                throw 'SERVER_ERROR_CANNOT_BACKUP_NLU_SERVER_FILE';
            return Promise.resolve(ssh.putFile(nluLocalFolder + '/' + model + '.md', modelFolder + '/data/nlu.md'));
        }).then(function (result) {
            var cmd = 'cat nlu.md';
            modelFolder = rootFolder + '/' + installationId + '::' + appId.toLowerCase();
            return ssh.execCommand(cmd, { cwd: modelFolder + '/data' });
        }).then(function (result) {
            if (result === null)
                throw 'SERVER_ERROR_NO_RESULT_FROM_RASA_SERVER';
            var updatedNluTextContent = '\n' + result.stdout + '\n';
            var regex = new RegExp("#{2}\\sintent:" + name + "\\s*$\n", 'm');
            var searchOnContent = updatedNluTextContent.match(regex);
            if (searchOnContent)
                return Promise.resolve(name);
            else
                throw 'SERVER_ERROR_COULDNOT_CREATE_INTENT';
        }).catch(function (error) {
            var errorMessage = typeof error === 'string' ? error : 'ERROR';
            logger.error('[RASAServiceImplementation::AddIntent] Error: ', error);
            if (error.code && error.code === 'ETIMEDOUT')
                errorMessage = 'SERVER_ERROR_TIMEOUT_FROM_RASA_SERVER';
            if (error.code && error.code === 'ECONNREFUSED')
                errorMessage = 'SERVER_ERROR_CONNECTION_REFUSED_FROM_RASA_SERVER';
            return Promise.reject(errorMessage);
        });
    };
    RASAServiceImplementation.prototype.DeleteAppIntent = function (subscriptionRegion, subscriptionKey, appId, intentId) {
        var _this = this;
        var installationId = '';
        var vcc = appId.split('::')[0];
        var model = appId.split('::')[1];
        var rootFolder = config_1.RASA_ROOT_FOLDER;
        var modelFolder = '';
        var nluLocalFolder = config_1.PATH_FOLDER_TEMP_NLU + '/' + vcc;
        var oldNluTextContent = '';
        var newNluTextContent = '';
        var ssh = null;
        return promises_shared_1.DummyPromise()
            .then(function (result) {
            return platform_service_1.GetPlatformVersion();
        }).then(function (result) {
            logger.info('[RASAServiceImplementation::DeleteAppIntent] Getting result:', result, 'after GetVersion');
            if (result) {
                installationId = result.installationId;
                return _this.GetSsh();
            }
            else {
                throw 'ERROR_NO_VERSION_DATA';
            }
        }).then(function (result) {
            ssh = result;
            var cmd = 'cat nlu.md';
            modelFolder = rootFolder + '/' + installationId + '::' + appId.toLowerCase();
            return ssh.execCommand(cmd, { cwd: modelFolder + '/data' });
        }).then(function (result) {
            if (result === null)
                throw 'SERVER_ERROR_NO_RESULT_FROM_RASA_SERVER';
            oldNluTextContent = '\n' + result.stdout + '\n';
            var regexToMatch = new RegExp("#{2}\\sintent:" + intentId + "\\s*$\n", 'm');
            var searchOnContent = oldNluTextContent.match(regexToMatch);
            if (!searchOnContent)
                throw 'SERVER_ERROR_INTENT_NOT_FOUND';
            var regexToReplace = new RegExp("(^#{2}\\sintent:" + intentId + "\\s*$\n)(^-.*$\n)*", 'm');
            newNluTextContent = oldNluTextContent.replace(regexToReplace, '');
            return file_manager_1.CreateFolder(nluLocalFolder);
        }).then(function (result) {
            if (!result)
                throw 'SERVER_ERROR_CANNOT_CREATE_NLU_TEMPORARY_FOLDER';
            var promiseArray = [];
            var tempModelPath = nluLocalFolder + '/' + model + '.md';
            promiseArray.push(file_manager_1.WriteFileContent(tempModelPath, newNluTextContent, 'utf-8'));
            var cmd = 'cp -R nlu.md nlu-backup.md';
            promiseArray.push(ssh.execCommand(cmd, { cwd: modelFolder + '/data' }));
            return Promise.all(promiseArray);
        }).then(function (result) {
            if (!result)
                throw 'SERVER_ERROR_DELETE_INTENT_FAILED';
            if (result && !result[0])
                throw 'SERVER_ERROR_CANNOT_CREATE_NLU_TEMPORARY_FILE';
            if (result && result[1] && result[1].stderr !== '')
                throw 'SERVER_ERROR_CANNOT_BACKUP_NLU_SERVER_FILE';
            return Promise.resolve(ssh.putFile(nluLocalFolder + '/' + model + '.md', modelFolder + '/data/nlu.md'));
        }).then(function (result) {
            var cmd = 'cat nlu.md';
            modelFolder = rootFolder + '/' + installationId + '::' + appId.toLowerCase();
            return ssh.execCommand(cmd, { cwd: modelFolder + '/data' });
        }).then(function (result) {
            if (result === null)
                throw 'SERVER_ERROR_NO_RESULT_FROM_RASA_SERVER';
            var updatedNluTextContent = result.stdout;
            var regex = new RegExp("#{2}\\sintent:" + intentId + "\\s*$\n", 'm');
            var searchOnContent = updatedNluTextContent.match(regex);
            if (!searchOnContent)
                return Promise.resolve(true);
            else
                throw 'SERVER_ERROR_COULDNOT_DELETE_INTENT';
        }).catch(function (error) {
            var errorMessage = typeof error === 'string' ? error : 'ERROR';
            logger.error('[RASAServiceImplementation::DeleteIntent] Error: ', error);
            if (error.code && error.code === 'ETIMEDOUT')
                errorMessage = 'SERVER_ERROR_TIMEOUT_FROM_RASA_SERVER';
            if (error.code && error.code === 'ECONNREFUSED')
                errorMessage = 'SERVER_ERROR_CONNECTION_REFUSED_FROM_RASA_SERVER';
            return Promise.reject(errorMessage);
        });
    };
    RASAServiceImplementation.prototype.GetIntentsList = function (subscriptionRegion, subscriptionKey, appId) {
        var _this = this;
        var installationId = '';
        var rootFolder = config_1.RASA_ROOT_FOLDER;
        var ssh = null;
        return promises_shared_1.DummyPromise()
            .then(function (result) {
            return platform_service_1.GetPlatformVersion();
        }).then(function (result) {
            logger.info('[RASAServiceImplementation::GetIntentsList] Getting result:', result, 'after GetVersion');
            if (result) {
                installationId = result.installationId;
                return _this.GetSsh();
            }
            else {
                throw 'ERROR_NO_VERSION_DATA';
            }
        }).then(function (result) {
            ssh = result;
            var cmd = 'cat nlu.md';
            var modelFolder = installationId + '::' + appId.toLowerCase();
            return ssh.execCommand(cmd, { cwd: rootFolder + '/' + modelFolder + '/data' });
        }).then(function (result) {
            if (result === null)
                throw 'SERVER_ERROR_NO_RESULT_FROM_RASA_SERVER';
            var arrIntentItems = result.stdout.split(/\r?\n/);
            var arrIntents = arrIntentItems.filter(function (aii) { return aii.includes('## intent:'); });
            var res = arrIntents ? arrIntents.map(function (ai) { return ai.replace('## intent:', ''); }) : [];
            var arrRes = [];
            res.map(function (r) {
                arrRes.push({
                    id: r,
                    name: r
                });
            });
            return Promise.resolve(arrRes);
        }).catch(function (error) {
            var errorMessage = typeof error === 'string' ? error : 'ERROR';
            logger.error('[RASAServiceImplementation::GetIntentsList] Error: ', error);
            if (error.code && error.code === 'ETIMEDOUT')
                errorMessage = 'SERVER_ERROR_TIMEOUT_FROM_RASA_SERVER';
            if (error.code && error.code === 'ECONNREFUSED')
                errorMessage = 'SERVER_ERROR_CONNECTION_REFUSED_FROM_RASA_SERVER';
            return Promise.reject(errorMessage);
        });
    };
    RASAServiceImplementation.prototype.AddUtterance = function (subscriptionRegion, subscriptionKey, appId, utterance) {
        var _this = this;
        var installationId = '';
        var vcc = utterance.appId.split('::')[0];
        var model = utterance.appId.split('::')[1];
        var intent = utterance.intent;
        var rootFolder = config_1.RASA_ROOT_FOLDER;
        var modelFolder = '';
        var nluLocalFolder = config_1.PATH_FOLDER_TEMP_NLU + '/' + vcc;
        var oldNluTextContent = '';
        var newNluTextContent = '';
        var ssh = null;
        return promises_shared_1.DummyPromise()
            .then(function (result) {
            return platform_service_1.GetPlatformVersion();
        }).then(function (result) {
            logger.info('[RASAServiceImplementation::AddIntent] Getting result:', result, 'after GetVersion');
            if (result) {
                installationId = result.installationId;
                return _this.GetSsh();
            }
            else {
                throw 'ERROR_NO_VERSION_DATA';
            }
        }).then(function (result) {
            ssh = result;
            var cmd = 'cat nlu.md';
            modelFolder = rootFolder + '/' + installationId + '::' + appId.toLowerCase();
            return ssh.execCommand(cmd, { cwd: modelFolder + '/data' });
        }).then(function (result) {
            var cmd = 'cat nlu.md';
            modelFolder = rootFolder + '/' + installationId + '::' + appId.toLowerCase();
            return ssh.execCommand(cmd, { cwd: modelFolder + '/data' });
        }).then(function (result) {
            if (result === null)
                throw 'SERVER_ERROR_NO_RESULT_FROM_RASA_SERVER';
            oldNluTextContent = '\n' + result.stdout + '\n\n';
            var matchRegex = new RegExp("(?<=(#{2}\\sintent:" + intent + ".*$\n)(^-.*$\n)*)(^- " + utterance.name + "\\s*$)", 'm');
            var searchOnContent = oldNluTextContent.match(matchRegex);
            if (searchOnContent)
                throw 'SERVER_ERROR_UTTERANCE_ALREADY_EXISTS';
            var replaceRegex = new RegExp("(?<=(#{2}\\sintent:" + intent + ".*$\n)(^-.*$\n)*)(^$)", 'm');
            newNluTextContent = oldNluTextContent.replace(replaceRegex, '- ' + utterance.name + '\n');
            return file_manager_1.CreateFolder(nluLocalFolder);
        }).then(function (result) {
            if (!result)
                throw 'SERVER_ERROR_CANNOT_CREATE_NLU_TEMPORARY_FOLDER';
            var promiseArray = [];
            var tempModelPath = nluLocalFolder + '/' + model + '.md';
            promiseArray.push(file_manager_1.WriteFileContent(tempModelPath, newNluTextContent, 'utf-8'));
            var cmd = 'cp -R nlu.md nlu-backup.md';
            promiseArray.push(ssh.execCommand(cmd, { cwd: modelFolder + '/data' }));
            return Promise.all(promiseArray);
        }).then(function (result) {
            if (!result)
                throw 'SERVER_ERROR_CREATE_UTTERANCE_FAILED';
            if (result && !result[0])
                throw 'SERVER_ERROR_CANNOT_CREATE_NLU_TEMPORARY_FILE';
            if (result && result[1] && result[1].stderr !== '')
                throw 'SERVER_ERROR_CANNOT_BACKUP_NLU_SERVER_FILE';
            return Promise.resolve(ssh.putFile(nluLocalFolder + '/' + model + '.md', modelFolder + '/data/nlu.md'));
        }).then(function (result) {
            var cmd = 'cat nlu.md';
            modelFolder = rootFolder + '/' + installationId + '::' + appId.toLowerCase();
            return ssh.execCommand(cmd, { cwd: modelFolder + '/data' });
        }).then(function (result) {
            if (result === null)
                throw 'SERVER_ERROR_NO_RESULT_FROM_RASA_SERVER';
            var updatedNluTextContent = '\n' + result.stdout + '\n';
            var matchRegex = new RegExp("(?<=(#{2}\\sintent:" + intent + ".*$\n)(^-.*$\n)*)(^-\\s" + _this.GetStringToRegex(utterance.name) + "\\s*$)", 'm');
            var searchOnContent = updatedNluTextContent.match(matchRegex);
            if (searchOnContent) {
                return Promise.resolve({
                    UtteranceText: utterance.name,
                    ExampleId: Math.round(Math.random() * 1000000)
                });
            }
            else
                throw 'SERVER_ERROR_COULDNOT_CREATE_UTTERANCE';
        }).catch(function (error) {
            var errorMessage = typeof error === 'string' ? error : 'ERROR';
            logger.error('[RASAServiceImplementation::AddUtterance] Error: ', error);
            if (error.code && error.code === 'ETIMEDOUT')
                errorMessage = 'SERVER_ERROR_TIMEOUT_FROM_RASA_SERVER';
            if (error.code && error.code === 'ECONNREFUSED')
                errorMessage = 'SERVER_ERROR_CONNECTION_REFUSED_FROM_RASA_SERVER';
            return Promise.reject(errorMessage);
        });
    };
    RASAServiceImplementation.prototype.DeleteAppUtterance = function (subscriptionRegion, subscriptionKey, appId, intentId, utteranceId) {
        var _this = this;
        var installationId = '';
        var vcc = appId.split('::')[0];
        var model = appId.split('::')[1];
        var rootFolder = config_1.RASA_ROOT_FOLDER;
        var modelFolder = '';
        var nluLocalFolder = config_1.PATH_FOLDER_TEMP_NLU + '/' + vcc;
        var oldNluTextContent = '';
        var newNluTextContent = '';
        var ssh = null;
        return promises_shared_1.DummyPromise()
            .then(function (result) {
            return platform_service_1.GetPlatformVersion();
        }).then(function (result) {
            logger.info('[RASAServiceImplementation::AddIntent] Getting result:', result, 'after GetVersion');
            if (result) {
                installationId = result.installationId;
                return _this.GetSsh();
            }
            else {
                throw 'ERROR_NO_VERSION_DATA';
            }
        }).then(function (result) {
            ssh = result;
            var cmd = 'cat nlu.md';
            modelFolder = rootFolder + '/' + installationId + '::' + appId.toLowerCase();
            return ssh.execCommand(cmd, { cwd: modelFolder + '/data' });
        }).then(function (result) {
            if (result === null)
                throw 'SERVER_ERROR_NO_RESULT_FROM_RASA_SERVER';
            oldNluTextContent = result.stdout;
            var regex = new RegExp("(?<=(#{2}\\sintent:" + intentId + "\\s*$\n)(^-.*$\n)*)(^- " + _this.GetStringToRegex(utteranceId) + "\\s*\n)", 'm');
            var searchOnContent = oldNluTextContent.match(regex);
            if (!searchOnContent)
                throw 'SERVER_ERROR_UTTERANCE_NOT_FOUND';
            newNluTextContent = oldNluTextContent.replace(regex, '');
            return file_manager_1.CreateFolder(nluLocalFolder);
        }).then(function (result) {
            if (!result)
                throw 'SERVER_ERROR_CANNOT_CREATE_NLU_TEMPORARY_FOLDER';
            var promiseArray = [];
            var tempModelPath = nluLocalFolder + '/' + model + '.md';
            promiseArray.push(file_manager_1.WriteFileContent(tempModelPath, newNluTextContent, 'utf-8'));
            var cmd = 'cp -R nlu.md nlu-backup.md';
            promiseArray.push(ssh.execCommand(cmd, { cwd: modelFolder + '/data' }));
            return Promise.all(promiseArray);
        }).then(function (result) {
            if (!result)
                throw 'SERVER_ERROR_CREATE_INTENT_FAILED';
            if (result && !result[0])
                throw 'SERVER_ERROR_CANNOT_CREATE_NLU_TEMPORARY_FILE';
            if (result && result[1] && result[1].stderr !== '')
                throw 'SERVER_ERROR_CANNOT_BACKUP_NLU_SERVER_FILE';
            return Promise.resolve(ssh.putFile(nluLocalFolder + '/' + model + '.md', modelFolder + '/data/nlu.md'));
        }).then(function (result) {
            var cmd = 'cat nlu.md';
            modelFolder = rootFolder + '/' + installationId + '::' + appId.toLowerCase();
            return ssh.execCommand(cmd, { cwd: modelFolder + '/data' });
        }).then(function (result) {
            if (result === null)
                throw 'SERVER_ERROR_NO_RESULT_FROM_RASA_SERVER';
            var updatedNluTextContent = result.stdout;
            var regex = new RegExp("(?<=(#{2}\\sintent:" + intentId + "\\s*$\n)(^-.*$\n)*)(^- " + _this.GetStringToRegex(utteranceId) + "\\s*$\n)", 'm');
            var searchOnContent = updatedNluTextContent.match(regex);
            if (!searchOnContent) {
                return Promise.resolve(_this.GetAllUtterancesFromRASAContent(updatedNluTextContent));
            }
            else
                throw 'SERVER_ERROR_COULDNOT_DELETE_UTTERANCE';
        }).catch(function (error) {
            var errorMessage = typeof error === 'string' ? error : 'ERROR';
            logger.error('[RASAServiceImplementation::DeleteUtterance] Error: ', error);
            if (error.code && error.code === 'ETIMEDOUT')
                errorMessage = 'SERVER_ERROR_TIMEOUT_FROM_RASA_SERVER';
            if (error.code && error.code === 'ECONNREFUSED')
                errorMessage = 'SERVER_ERROR_CONNECTION_REFUSED_FROM_RASA_SERVER';
            return Promise.reject(errorMessage);
        });
    };
    RASAServiceImplementation.prototype.GetUtterancesList = function (subscriptionRegion, subscriptionKey, appId) {
        var _this = this;
        var installationId = '';
        var rootFolder = config_1.RASA_ROOT_FOLDER;
        var ssh = null;
        return promises_shared_1.DummyPromise()
            .then(function (result) {
            return platform_service_1.GetPlatformVersion();
        }).then(function (result) {
            logger.info('[RASAServiceImplementation::GetIntentsList] Getting result:', result, 'after GetVersion');
            if (result) {
                installationId = result.installationId;
                return _this.GetSsh();
            }
            else {
                throw 'ERROR_NO_VERSION_DATA';
            }
        }).then(function (result) {
            ssh = result;
            var cmd = 'cat nlu.md';
            var modelFolder = installationId + '::' + appId.toLowerCase();
            return ssh.execCommand(cmd, { cwd: rootFolder + '/' + modelFolder + '/data' });
        }).then(function (result) {
            if (result === null)
                throw 'SERVER_ERROR_NO_RESULT_FROM_RASA_SERVER';
            var nluTextContent = result.stdout;
            return Promise.resolve(_this.GetAllUtterancesFromRASAContent(nluTextContent));
        }).catch(function (error) {
            var errorMessage = typeof error === 'string' ? error : 'ERROR';
            logger.error('[RASAServiceImplementation::GetUtterancesList] Error: ', error);
            if (error.code && error.code === 'ETIMEDOUT')
                errorMessage = 'SERVER_ERROR_TIMEOUT_FROM_RASA_SERVER';
            if (error.code && error.code === 'ECONNREFUSED')
                errorMessage = 'SERVER_ERROR_CONNECTION_REFUSED_FROM_RASA_SERVER';
            return Promise.reject(errorMessage);
        });
    };
    RASAServiceImplementation.prototype.AddSimpleEntity = function (subscriptionRegion, subscriptionKey, appId, entity) {
        return null;
    };
    RASAServiceImplementation.prototype.RenameEntity = function (subscriptionRegion, subscriptionKey, appId, entityId, entityName) {
        return null;
    };
    RASAServiceImplementation.prototype.GetEntitiesList = function (subscriptionRegion, subscriptionKey, appId) {
        return promises_shared_1.DummyPromise()
            .then(function (result) {
            return Promise.resolve([]);
        });
    };
    RASAServiceImplementation.prototype.GetEntity = function (subscriptionRegion, subscriptionKey, appId, entityId) {
        return null;
    };
    RASAServiceImplementation.prototype.DeleteAppEntity = function (subscriptionRegion, subscriptionKey, appId, entityId) {
        return null;
    };
    RASAServiceImplementation.prototype.TrainAppManual = function (subscriptionRegion, subscriptionKey, appId) {
        var _this = this;
        var installationId = '';
        var vcc = appId.split('::')[0];
        var model = appId.split('::')[1];
        var rootFolder = config_1.RASA_ROOT_FOLDER;
        var modelFolder = '';
        var ssh = null;
        return promises_shared_1.DummyPromise()
            .then(function (result) {
            return platform_service_1.GetPlatformVersion();
        }).then(function (result) {
            logger.info('[RASAServiceImplementation::AddApp] Getting result:', result, 'after GetVersion');
            if (result) {
                installationId = result.installationId;
                return _this.GetSsh();
            }
            else {
                throw 'SERVER_ERROR_NO_VERSION_DATA';
            }
        }).then(function (result) {
            if (!result)
                throw 'SERVER_ERROR_CANNOT_CONNECT_RASA_SERVER';
            ssh = result;
            var cmd = 'rasa train nlu -c config.yml --out models';
            modelFolder = rootFolder + '/' + installationId + '::' + appId.toLowerCase();
            return ssh.execCommand(cmd, { cwd: modelFolder });
        }).then(function (result) {
            if (!result)
                throw 'SERVER_ERROR_COULDNOT_TRAIN_MODEL';
            if (result && result.stdout) {
                var matchRegex = new RegExp("Your Rasa model is trained and saved ", 'm');
                var searchOnContent = result.stdout.match(matchRegex);
                if (searchOnContent) {
                    return Promise.resolve(true);
                }
                else
                    throw 'SERVER_ERROR_MODEL_TRAIN_FAILED';
            }
            else {
                throw 'SERVER_ERROR_MODEL_TRAIN_FAILED';
            }
        }).catch(function (error) {
            var errorMessage = typeof error === 'string' ? error : 'ERROR';
            logger.error('[RASAServiceImplementation::TrainApp] Error: ', error);
            if (error.code && error.code === 'ETIMEDOUT')
                errorMessage = 'SERVER_ERROR_TIMEOUT_FROM_RASA_SERVER';
            if (error.code && error.code === 'ECONNREFUSED')
                errorMessage = 'SERVER_ERROR_CONNECTION_REFUSED_FROM_RASA_SERVER';
            return Promise.reject(errorMessage);
        });
    };
    RASAServiceImplementation.prototype.TrainApp = function (subscriptionRegion, subscriptionKey, appId) {
        var _this = this;
        var installationId = '';
        var vcc = appId.split('::')[0];
        var model = appId.split('::')[1];
        var rootFolder = config_1.RASA_ROOT_FOLDER;
        var modelFolder = '';
        var modelPort = null;
        var modelTrainFile = null;
        var ssh = null;
        var runModelCommand = null;
        return promises_shared_1.DummyPromise()
            .then(function (result) {
            return platform_service_1.GetPlatformVersion();
        }).then(function (result) {
            logger.info('[RASAServiceImplementation::TrainApp] Getting result:', result, 'after GetVersion');
            if (result) {
                installationId = result.installationId;
                modelFolder = config_1.RASA_ROOT_FOLDER + '/' + installationId + '::' + appId.toLowerCase();
                return _this.GetSsh();
            }
            else {
                throw 'SERVER_ERROR_NO_VERSION_DATA';
            }
        }).then(function (result) {
            if (!result)
                throw 'SERVER_ERROR_CANNOT_CONNECT_RASA_SERVER';
            ssh = result;
            var cmd = 'cat nlu.md';
            var modelFolder = installationId + '::' + appId.toLowerCase();
            return ssh.execCommand(cmd, { cwd: rootFolder + '/' + modelFolder + '/data' });
        }).then(function (result) {
            if (result === null)
                throw 'SERVER_ERROR_NO_RESULT_FROM_RASA_SERVER';
            var nluTextContent = result.stdout;
            var arrIntentItems = _this.GetAllIntentsFromRASAContent(nluTextContent);
            var arrUtteranceItems = _this.GetAllUtterancesFromRASAContent(nluTextContent);
            arrIntentItems.map(function (aii) {
                if (arrUtteranceItems.filter(function (aui) { return aui.intentLabel === aii.id; }).length === 0)
                    throw 'SERVER_ERROR_MISSING_UTTERANCES_TO_TRAIN';
            });
            var cmd = 'python3 trainDataGenerator.py';
            return ssh.execCommand(cmd, { cwd: modelFolder });
        }).then(function (result) {
            if (!result)
                throw 'SERVER_ERROR_CANNOT_GENERATE_TRAIN_DATA_FILE';
            return _this.RunApp(ssh, installationId + '::' + appId.toLowerCase(), 'TRAIN');
        }).then(function (result) {
            if (!result)
                throw 'SERVER_ERROR_CANNOT_RUN_APP';
            modelPort = result;
            var cmd = 'curl localhost:' + modelPort + '/status';
            return ssh.execCommand(cmd, { cwd: modelFolder });
        }).then(function (result) {
            if (result && result.stderr && result.stderr.includes('Connection refused')) {
                redis_service_1.Redis.DeleteRunningRasaModel(installationId + '::' + appId.toLowerCase());
                throw 'SERVER_ERROR_CANNOT_RUN_APP';
            }
            modelTrainFile = 'nlu-' + moment.utc().format('YYYYMMDD-HHmmss') + '.tar.gz';
            var cmd = 'curl -X POST http://localhost:' + modelPort + '/model/train -H \'Accept-Encoding: gzip, deflate\' -H \'Content-Type: application/json\' -d @' + modelFolder + '/trainData.json --output ' + modelFolder + '/models/' + modelTrainFile;
            return ssh.execCommand(cmd, { cwd: modelFolder });
        }).then(function (result) {
            var cmd = 'ls';
            return ssh.execCommand(cmd, { cwd: modelFolder + '/models' });
        }).then(function (result) {
            var arrNluFiles = result && result.stdout ? result.stdout.split('\n') : [];
            if (!arrNluFiles.includes(modelTrainFile))
                throw 'SERVER_ERROR_TRAIN_MODEL_FAILED';
            return _this.StopApp(ssh, appId, modelPort, 'TRAIN');
        }).then(function (result) {
            return Promise.resolve({ status: 'UpToDate', statusId: 2 });
        }).catch(function (error) {
            var errorMessage = typeof error === 'string' ? error : 'ERROR';
            logger.error('[RASAServiceImplementation::TrainApp] Error: ', error);
            if (error === 'SERVER_ERROR_MISSING_UTTERANCES_TO_TRAIN')
                return Promise.resolve({ status: 'Queued', statusId: 9 });
            if (error.code && error.code === 'ETIMEDOUT')
                errorMessage = 'SERVER_ERROR_TIMEOUT_FROM_RASA_SERVER';
            if (error.code && error.code === 'ECONNREFUSED')
                errorMessage = 'SERVER_ERROR_CONNECTION_REFUSED_FROM_RASA_SERVER';
            return Promise.reject(errorMessage);
        });
    };
    RASAServiceImplementation.prototype.ModelQuery = function (subscriptionRegion, subscriptionKey, appId, query, processAudios, nluName, resolve) {
        var installationId = '';
        var vcc = appId.split('::')[0];
        var model = appId.split('::')[1];
        var rootFolder = config_1.RASA_ROOT_FOLDER;
        var modelFolder = '';
        var modelPort = null;
        return promises_shared_1.DummyPromise()
            .then(function (result) {
            return platform_service_1.GetPlatformVersion();
        }).then(function (result) {
            logger.info('[RASAServiceImplementation::ModelQuery] Getting result:', result, 'after GetVersion');
            if (result) {
                installationId = result.installationId;
                modelFolder = rootFolder + '/' + installationId + '::' + appId.toLowerCase();
                return redis_service_1.Redis.GetRunningRasaModel(installationId + '::' + appId.toLowerCase());
            }
            else {
                throw 'SERVER_ERROR_NO_VERSION_DATA';
            }
        })
            .then(function (result) {
            if (!result)
                throw 'SERVER_ERROR_CANNOT_GET_MODEL_PORT';
            modelPort = result.port;
            var getOptions = {
                uri: "http://" + config_1.RASA_IP + ":" + modelPort + "/status",
                method: 'GET',
                strictSSL: false
            };
            return rp(getOptions);
        }).then(function (result) {
            logger.debug('[RASAServiceImplementation::ModelQuery] Status RESULT: ', result);
            if (result && result.includes('refused')) {
                redis_service_1.Redis.DeleteRunningRasaModel(installationId + '::' + appId.toLowerCase());
                throw 'SERVER_ERROR_CANNOT_RUN_APP';
            }
            var getOptions = {
                uri: "http://" + config_1.RASA_IP + ":" + modelPort + "/model/parse",
                method: 'POST',
                body: {
                    text: query.replace('"', '\"')
                },
                json: true,
                strictSSL: false
            };
            logger.info('[RASAServiceImplementation::ModelQuery] getting getOptions:', getOptions, 'appId:', appId, 'query:', query, 'processAudios:', processAudios, 'nluName:', nluName);
            return rp(getOptions);
        }).then(function (result) {
            if (!result)
                throw 'SERVER_ERROR_PARSE_REQUEST_FAILED';
            logger.info('[RASAServiceImplementation::ModelQuery] Result: ' + JSON.stringify(result));
            var intents = [];
            result.intent_ranking.map(function (ir) {
                intents.push({
                    intent: ir.name,
                    score: ir.confidence
                });
            });
            var parseResult = {
                query: result.text ? result.text : null,
                topScoringIntent: {
                    intent: result.intent && result.intent.name ? result.intent.name : null,
                    score: result.intent && result.intent.confidence ? result.intent.confidence : 0
                },
                intents: intents,
                entities: []
            };
            logger.info('[RASAServiceImplementation::ModelQuery] FormattedResult: ' + JSON.stringify(parseResult));
            return Promise.resolve(parseResult);
        }).catch(function (error) {
            var errorMessage = typeof error === 'string' ? error : 'ERROR';
            logger.error('[RASAServiceImplementation::ModelQuery] Error: ', error);
            if (error.code && error.code === 'ETIMEDOUT')
                errorMessage = 'SERVER_ERROR_TIMEOUT_FROM_RASA_SERVER';
            if (error.code && error.code === 'ECONNREFUSED')
                errorMessage = 'SERVER_ERROR_CONNECTION_REFUSED_FROM_RASA_SERVER';
            return Promise.reject(errorMessage);
        });
    };
    RASAServiceImplementation.prototype.PublishApp = function (subscriptionRegion, subscriptionKey, appId, isStaging) {
        if (isStaging)
            return this.RunApp(null, appId, 'PARSE');
        else
            return this.StopApp(null, appId, 0, 'PARSE');
    };
    RASAServiceImplementation.prototype.ExportModel = function (nlu, res) {
        var _this = this;
        var installationId = '';
        var rootFolder = config_1.RASA_ROOT_FOLDER;
        var modelFolder = null;
        var ssh = null;
        return promises_shared_1.DummyPromise().then(function (result) {
            return platform_service_1.GetPlatformVersion();
        }).then(function (result) {
            logger.info('[RASAServiceImplementation::GetAppsList] Getting result:', result, 'after GetVersion');
            if (result) {
                installationId = result.installationId;
                return _this.GetSsh();
            }
            else {
                throw 'ERROR_NO_VERSION_DATA';
            }
        }).then(function (result) {
            ssh = result;
            var cmd = 'cat nlu.md';
            modelFolder = rootFolder + '/' + installationId + '::' + nlu.vcc + '::' + nlu.name.toLowerCase();
            return ssh.execCommand(cmd, { cwd: modelFolder + '/data' });
        }).then(function (result) {
            if (result === null)
                throw 'SERVER_ERROR_NO_RESULT_FROM_RASA_SERVER';
            if (result.stdout && result.stdout.includes('## intent:None\n- xxxxx'))
                result.stdout = result.stdout.replace('## intent:None\n- xxxxx', '');
            return Promise.resolve(res.end(result.stdout));
        }).catch(function (error) {
            var errorMessage = typeof error === 'string' ? error : 'ERROR';
            logger.error('[RASAServiceImplementation::ExportModel] Error: ', error);
            if (error.code && error.code === 'ETIMEDOUT')
                errorMessage = 'SERVER_ERROR_TIMEOUT_FROM_RASA_SERVER';
            if (error.code && error.code === 'ECONNREFUSED')
                errorMessage = 'SERVER_ERROR_CONNECTION_REFUSED_FROM_RASA_SERVER';
            return Promise.reject(errorMessage);
        });
    };
    RASAServiceImplementation.prototype.GetIntentsListDemo = function (subscriptionRegion, subscriptionKey, appId, resolve, reject) {
        var _this = this;
        var installationId = '';
        var rootFolder = config_1.RASA_ROOT_FOLDER;
        var ssh = null;
        return promises_shared_1.DummyPromise()
            .then(function (result) {
            return platform_service_1.GetPlatformVersion();
        }).then(function (result) {
            logger.info('[RASAServiceImplementation::GetIntentsListDemo] Getting result:', result, 'after GetVersion');
            if (result) {
                installationId = result.installationId;
                return _this.GetSsh();
            }
            else {
                throw 'ERROR_NO_VERSION_DATA';
            }
        }).then(function (result) {
            ssh = result;
            var cmd = 'cat nlu.md';
            var modelFolder = installationId + '::' + appId.toLowerCase();
            return ssh.execCommand(cmd, { cwd: rootFolder + '/' + modelFolder + '/data' });
        }).then(function (result) {
            if (result === null)
                throw 'SERVER_ERROR_NO_RESULT_FROM_RASA_SERVER';
            var arrIntentItems = result.stdout.split(/\r?\n/);
            var arrIntents = arrIntentItems.filter(function (aii) { return aii.includes('## intent:'); });
            var res = arrIntents ? arrIntents.map(function (ai) { return ai.replace('## intent:', ''); }) : [];
            var arrRes = [];
            res.map(function (r) {
                arrRes.push({
                    id: r,
                    name: r
                });
            });
            return Promise.resolve(arrRes);
        }).catch(function (error) {
            var errorMessage = typeof error === 'string' ? error : 'ERROR';
            logger.error('[RASAServiceImplementation::GetIntentsListDemo] Error: ', error);
            if (error.code && error.code === 'ETIMEDOUT')
                errorMessage = 'SERVER_ERROR_TIMEOUT_FROM_RASA_SERVER';
            if (error.code && error.code === 'ECONNREFUSED')
                errorMessage = 'SERVER_ERROR_CONNECTION_REFUSED_FROM_RASA_SERVER';
            return Promise.reject(errorMessage);
        });
    };
    RASAServiceImplementation.prototype.GetTrainApp = function (subscriptionRegion, subscriptionKey, appId) {
        var _this = this;
        var installationId = '';
        var rootFolder = config_1.RASA_ROOT_FOLDER;
        var ssh = null;
        return promises_shared_1.DummyPromise()
            .then(function (result) {
            return platform_service_1.GetPlatformVersion();
        }).then(function (result) {
            logger.info('[RASAServiceImplementation::GetIntentsList] Getting result:', result, 'after GetVersion');
            if (result) {
                installationId = result.installationId;
                return _this.GetSsh();
            }
            else {
                throw 'ERROR_NO_VERSION_DATA';
            }
        }).then(function (result) {
            ssh = result;
            var cmd = 'cat nlu.md';
            var modelFolder = installationId + '::' + appId.toLowerCase();
            return ssh.execCommand(cmd, { cwd: rootFolder + '/' + modelFolder + '/data' });
        }).then(function (result) {
            if (result === null)
                throw 'SERVER_ERROR_NO_RESULT_FROM_RASA_SERVER';
            var arrIntentItems = _this.GetAllIntentsFromRASAContent(result.stdout);
            var arrUtteranceItems = _this.GetAllUtterancesFromRASAContent(result.stdout);
            var arrRes = [];
            arrIntentItems.map(function (aii) {
                var intentDetails = {
                    exampleCount: 0,
                    failureReason: arrUtteranceItems.filter(function (aui) { return aui.intentLabel === aii.id; }).length === 0 ? 'FewLabels' : 'Aborted',
                    status: 'Fail',
                    statusId: 1
                };
                arrRes.push({ details: intentDetails, modelId: aii.id });
            });
            return Promise.resolve(arrRes);
        }).catch(function (error) {
            var errorMessage = typeof error === 'string' ? error : 'ERROR';
            logger.error('[RASAServiceImplementation::GetIntentsList] Error: ', error);
            if (error.code && error.code === 'ETIMEDOUT')
                errorMessage = 'SERVER_ERROR_TIMEOUT_FROM_RASA_SERVER';
            if (error.code && error.code === 'ECONNREFUSED')
                errorMessage = 'SERVER_ERROR_CONNECTION_REFUSED_FROM_RASA_SERVER';
            return Promise.reject(errorMessage);
        });
    };
    RASAServiceImplementation.prototype.GetManagerToken = function (info) {
        return null;
    };
    RASAServiceImplementation.prototype.LoadAzureAccounts = function (subscriptionRegion, subscriptionKey, data) {
        return null;
    };
    RASAServiceImplementation.prototype.UnAssignSubscriptionKeyOfVCC = function (subscriptionRegion, subscriptionKey, appId, data) {
        return null;
    };
    RASAServiceImplementation.prototype.AssignSubscriptionKeyOfVCC = function (subscriptionRegion, subscriptionKey, appId, data) {
        return null;
    };
    RASAServiceImplementation.prototype.GetStringToRegex = function (text) {
        var specialCharacters = ['\\', '[', ']', '^', '$', '.', '|', '¿', '?', '¡', '!', '*', '+', '(', ')'];
        specialCharacters.map(function (sc) {
            text = text.split(sc).join('\\' + sc);
        });
        return text;
    };
    RASAServiceImplementation.prototype.GetSsh = function () {
        var _this = this;
        return promises_shared_1.DummyPromise()
            .then(function (result) {
            if (_this.mySsh && _this.mySsh.connection) {
                return Promise.resolve(_this.mySsh);
            }
            else {
                _this.mySsh = null;
                _this.mySsh = new NodeSSH();
                return _this.mySsh.connect({
                    host: config_1.RASA_IP,
                    username: config_1.RASA_USER,
                    password: config_1.RASA_PASSWORD,
                    port: config_1.RASA_PORT,
                    readyTimeout: 300000,
                    keepaliveCountMax: 10
                });
            }
        }).then(function (result) {
            logger.info('[RASAServiceImplementation::GetSsh] Getting result:', (result && result.connection ? true : false), 'after Connect');
            return Promise.resolve(_this.mySsh);
        }).catch(function (error) {
            var errorMessage = typeof error === 'string' ? error : 'ERROR';
            logger.error('[RASAServiceImplementation::GetSsh] Error: ', error);
            if (error.code && error.code === 'ETIMEDOUT')
                errorMessage = 'SERVER_ERROR_TIMEOUT_FROM_RASA_SERVER';
            if (error.code && error.code === 'ECONNREFUSED')
                errorMessage = 'SERVER_ERROR_CONNECTION_REFUSED_FROM_RASA_SERVER';
            return Promise.reject(errorMessage);
        });
    };
    RASAServiceImplementation.prototype.GetAllIntentsFromRASAContent = function (content) {
        var arrIntentItems = content.split(/\r?\n/);
        var arrIntents = arrIntentItems.filter(function (aii) { return aii.includes('## intent:'); });
        var res = arrIntents ? arrIntents.map(function (ai) { return ai.replace('## intent:', ''); }) : [];
        var arrRes = [];
        res.map(function (r) {
            if (r.trim() !== 'None') {
                arrRes.push({
                    id: r.trim(),
                    name: r.trim()
                });
            }
        });
        return arrRes;
    };
    RASAServiceImplementation.prototype.GetAllUtterancesFromRASAContent = function (content) {
        var arrIntentBlock = content.split(/##\sintent:/);
        var arrUterances = [];
        arrIntentBlock.map(function (aib) {
            if (aib !== '') {
                var arrIntentItems = aib.split(/\r?\n/);
                var intent_1 = arrIntentItems.shift();
                arrIntentItems.map(function (aii) {
                    if (aii !== '' && intent_1.trim() !== 'None') {
                        aii = aii.replace('- ', '');
                        arrUterances.push({
                            id: utilities_1.removeSpecialCharacters(utilities_1.removeAccents(aii.trim())),
                            intentLabel: intent_1.trim(),
                            text: aii.trim(),
                            intentPredictions: [
                                {
                                    name: intent_1.trim(),
                                    score: null
                                }
                            ]
                        });
                    }
                });
            }
        });
        return arrUterances;
    };
    RASAServiceImplementation.prototype.RunApp = function (ssh, appId, task) {
        var _this = this;
        logger.info('[RASAServiceImplementation::RunApp] Launching: ' + appId + ', task: ' + task);
        var modelFolder = config_1.RASA_ROOT_FOLDER + '/';
        var selectedPort = null;
        var runModelCommand = null;
        var resultMessage = null;
        return promises_shared_1.DummyPromise()
            .then(function (result) {
            if (appId.split('::').length === 2) {
                return platform_service_1.GetPlatformVersion();
            }
            else {
                return true;
            }
        }).then(function (result) {
            logger.info('[RASAServiceImplementation::RunApp] Getting result:', result, 'after GetVersion');
            if (appId.split('::').length === 2) {
                appId = result.installationId + '::' + appId.toLowerCase();
            }
            modelFolder += appId;
            if (ssh === null) {
                return _this.GetSsh();
            }
            else {
                return Promise.resolve(ssh);
            }
        }).then(function (result) {
            if (!result)
                throw 'SERVER_ERROR_GET_SSH_FAILED';
            ssh = result;
            if (task === 'TRAIN') {
                return _this.StopApp(ssh, appId, null, task);
            }
            else {
                return Promise.resolve(true);
            }
        }).then(function (result) {
            var cmd = 'for i in {' + config_1.RASA_AVAILABLE_PORTS + '}; do (exec 2>&-; echo > /dev/tcp/localhost/$i && echo $i is open); done';
            return ssh.execCommand(cmd, { cwd: modelFolder });
        }).then(function (result) {
            if (!result)
                throw 'SERVER_ERROR_CANNOT_GET_PORTS_LIST';
            var freePorts = [];
            var rangeAvailablePorts = config_1.RASA_AVAILABLE_PORTS.split('..');
            for (var _i = Number(rangeAvailablePorts[0]); _i <= Number(rangeAvailablePorts[1]); _i++) {
                if (result.stdout === '' || !result.stdout.includes(_i + ' is open'))
                    freePorts.push(_i);
            }
            if (freePorts.length === 0)
                throw 'SERVER_ERROR_AVAILABLE_PORTS_NOT_FOUND';
            selectedPort = freePorts[Math.floor(Math.random() * freePorts.length)];
            runModelCommand = 'nohup /usr/local/bin/rasa run --model ' + modelFolder + '/models --enable-api --log-file ' + modelFolder + '/' + task + '.log -p ' + selectedPort + ' < /dev/null > /dev/null 2>&1 &';
            logger.info('[RASAServiceImplementation::RunApp] Launching: ' + runModelCommand);
            ssh.execCommand('sudo su -', { cwd: modelFolder, stdin: runModelCommand + '\n', stdout: 'DONE', execOptions: { pty: true } });
            return promises_shared_1.SleepPromise(10);
        }).then(function (result) {
            return _this.CheckRunningApp(ssh, appId, selectedPort, task);
        }).then(function (result) {
            if (task === 'TRAIN') {
                return Promise.resolve(true);
            }
            else if (task === 'PARSE') {
                var runningModel = {
                    id: appId,
                    port: selectedPort,
                    created: moment.utc().format('YYYY-MM-DD HH:mm:ss')
                };
                return redis_service_1.Redis.SaveRunningRasaModel(appId, runningModel);
            }
        }).then(function (result) {
            return Promise.resolve(selectedPort);
        }).catch(function (error) {
            var errorMessage = typeof error === 'string' ? error : 'ERROR';
            logger.error('[RASAServiceImplementation::RunApp] Error: ', error, '=>', resultMessage);
            if (error.code && error.code === 'ETIMEDOUT')
                errorMessage = 'SERVER_ERROR_TIMEOUT_FROM_RASA_SERVER';
            if (error.code && error.code === 'ECONNREFUSED')
                errorMessage = 'SERVER_ERROR_CONNECTION_REFUSED_FROM_RASA_SERVER';
            return Promise.reject(errorMessage);
        });
    };
    RASAServiceImplementation.prototype.CheckRunningApp = function (ssh, appId, port, action, counter) {
        var _this = this;
        counter = counter ? counter : 0;
        var modelFolder = config_1.RASA_ROOT_FOLDER + '/' + appId;
        return promises_shared_1.SleepPromise(5)
            .then(function (result) {
            if (counter >= 10) {
                redis_service_1.Redis.DeleteRunningRasaModel(appId);
                throw 'SERVER_ERROR_RETRY_LIMIT_EXCEEDED';
            }
            var cmd = 'ls';
            return ssh.execCommand(cmd, { cwd: modelFolder });
        }).then(function (result) {
            if (result.stdout.split('\n').includes(action + '.log')) {
                return true;
            }
            else {
                return _this.CheckRunningApp(ssh, appId, port, action, counter + 1);
            }
        }).then(function (result) {
            var cmd = 'curl localhost:' + port + '/status';
            return ssh.execCommand(cmd, { cwd: modelFolder });
        }).then(function (result) {
            if (result && result.stderr && !result.stderr.includes('Connection refused')) {
                return Promise.resolve(true);
            }
            else {
                return _this.CheckRunningApp(ssh, appId, port, action, counter + 1);
            }
        }).catch(function (error) {
            var errorMessage = typeof error === 'string' ? error : 'ERROR';
            logger.error('[RASAServiceImplementation::CheckRunningApp] Error: ', error);
            if (error.code && error.code === 'ETIMEDOUT')
                errorMessage = 'SERVER_ERROR_TIMEOUT_FROM_RASA_SERVER';
            if (error.code && error.code === 'ECONNREFUSED')
                errorMessage = 'SERVER_ERROR_CONNECTION_REFUSED_FROM_RASA_SERVER';
            return Promise.reject(errorMessage);
        });
    };
    RASAServiceImplementation.prototype.StopApp = function (ssh, appId, port, task) {
        var _this = this;
        var modelFolder = config_1.RASA_ROOT_FOLDER + '/';
        logger.info('[RASAServiceImplementation::StopApp] Stopping: ' + appId + ', port: ' + port + ', task: ' + task);
        return promises_shared_1.DummyPromise()
            .then(function (result) {
            if (appId.split('::').length === 2) {
                return platform_service_1.GetPlatformVersion();
            }
            else {
                return true;
            }
        }).then(function (result) {
            logger.info('[RASAServiceImplementation::StopApp] Getting result:', result, 'after GetVersion');
            if (appId.split('::').length === 2) {
                appId = result.installationId + '::' + appId.toLowerCase();
            }
            modelFolder += appId;
            if (ssh === null) {
                return _this.GetSsh();
            }
            else {
                return Promise.resolve(ssh);
            }
        }).then(function (result) {
            if (!result)
                throw 'SERVER_ERROR_GET_SSH_FAILED';
            ssh = result;
            var cmd = 'ps -ax | grep ' + appId;
            return ssh.execCommand(cmd, { cwd: modelFolder });
        }).then(function (result) {
            if (!result)
                throw 'SERVER_ERROR_CANNOT_FOUND_RUNNING_PROCESSES';
            var arrAllRunningProcesses = result.stdout.split('\n');
            var arrAppRunningProcesses = arrAllRunningProcesses.filter(function (aarp) { return aarp.includes(appId); });
            var arrKillPid = [];
            arrAppRunningProcesses.map(function (aarp) {
                if (aarp.includes(task + '.log'))
                    arrKillPid.push(Number(aarp.trim().split(' ')[0]));
            });
            if (arrKillPid.length > 0) {
                var cmd = 'pkill -f "' + modelFolder + '/' + task + '.log';
                if (port)
                    cmd += ' -p ' + port;
                cmd += '" ';
                logger.info('[RASAServiceImplementation::StopApp] Stopping: ' + cmd);
                ssh.execCommand('sudo su -', { cwd: modelFolder, stdin: cmd + '\n', execOptions: { pty: true } });
                return promises_shared_1.SleepPromise(5);
            }
            else {
                return Promise.resolve(true);
            }
        }).then(function (result) {
            return Promise.resolve(redis_service_1.Redis.DeleteRunningRasaModel(appId));
        }).catch(function (error) {
            var errorMessage = typeof error === 'string' ? error : 'ERROR';
            logger.error('[RASAServiceImplementation::StopApp] Error: ', error);
            if (error.code && error.code === 'ETIMEDOUT')
                errorMessage = 'SERVER_ERROR_TIMEOUT_FROM_RASA_SERVER';
            if (error.code && error.code === 'ECONNREFUSED')
                errorMessage = 'SERVER_ERROR_CONNECTION_REFUSED_FROM_RASA_SERVER';
            return Promise.reject(errorMessage);
        });
    };
    RASAServiceImplementation.prototype.ImportNLUFromRASA = function (nlu) {
        var nluContent = null;
        var nluFileRedisKey = 'NLU_FILE:' + nlu.createdByUserId;
        var tempModelNLUPath = config_1.PATH_FOLDER_TEMP_NLU + '/' + nlu.vcc + '/' + nlu.name + '-nlu.md';
        return promises_shared_1.DummyPromise().then(function (result) {
            return redis_service_1.Redis.GetFileNLU(nlu.vcc, (nluFileRedisKey));
        }).then(function (result) {
            if (!result)
                throw 'SERVER_ERROR_NLU_REDIS_DATA_NOT_FOUND';
            if (result.data && !result.data.includes('## intent:None'))
                result.data += '\n## intent:None\n- xxxxx\n\n';
            nluContent = result.data;
            return file_manager_1.WriteFileContent(tempModelNLUPath, nluContent, 'utf-8');
        }).then(function (result) {
            if (!result)
                throw 'SERVER_ERROR_CANNOT_CREATE_NLU_FILE';
            return Promise.resolve(nluContent);
        }).catch(function (error) {
            var errorMessage = typeof error === 'string' ? error : 'ERROR';
            logger.error('[RASAServiceImplementation::ImportNLUFromRASA] Error: ', error);
            return Promise.reject(errorMessage);
        });
    };
    return RASAServiceImplementation;
}());
exports.RASAServiceImplementation = RASAServiceImplementation;
