"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var moment = require("moment");
var log4js = require("log4js");
var mssql_service_1 = require("../db/mssql.service");
var redis_service_1 = require("../db/redis.service");
var promises_shared_1 = require("../shared/promises.shared");
var config_1 = require("../config/config");
var file_manager_1 = require("../shared/file-manager");
var utilities_1 = require("../shared/utilities");
var intent_service_1 = require("./intent.service");
var cognitive_analysis_implementation_1 = require("../implementation/cognitive-analysis.implementation");
var validate_shared_1 = require("../shared/validate.shared");
var logger = log4js.getLogger('ServerScripts');
function NLUService(app) {
    var cognitiveService;
    app.post('/api/nlu/validate_azure_credentials', function (req, res, nex) {
        logger.info('[NLUService::validate_azure_credentials] Received ' + req.method + ' method on \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var azureCredentials = req.body.azureSubscription;
        var azureSubscriptionKey = req.body.azureSubscriptionKey;
        var tenantVCC = {};
        var authObject;
        promises_shared_1.DummyPromise()
            .then(function (result) {
            return cognitive_analysis_implementation_1.GetCognitiveService(null, 'LUIS');
        }).then(function (result) {
            cognitiveService = result;
            return mssql_service_1.MSSql.GetTenantVCCByVCC(vcc);
        })
            .then(function (result) {
            if (!result)
                throw 'SERVER_ERROR_TENANT_NOT_FOUND';
            tenantVCC = result;
            var info = {
                tenant_id: tenantVCC.tenantId,
                client_id: tenantVCC.clientId,
                client_secret: tenantVCC.clientSecret,
                resource: tenantVCC.resource
            };
            return cognitiveService.GetManagerToken(info);
        })
            .then(function (result) {
            if (!result)
                throw 'SERVER_ERROR_NLU_TOKEN_GENERATOR_FAILED';
            authObject = result;
            return cognitiveService.LoadAzureAccounts(azureSubscriptionKey.subscriptionRegion, azureSubscriptionKey.subscriptionKey, authObject);
        })
            .then(function (result) {
            if (!result)
                throw 'SERVER_ERROR_NLU_INVALID_SUBSCRIPTIONKEY';
            if (result && result.length === 0)
                throw 'SERVER_ERROR_NLU_ACCOUNTS_NOT_FOUND';
            var isValid = false;
            var resultDescription;
            var validResult = result.filter(function (r) { return r.AzureSubscriptionId === azureCredentials.azureSubscription; });
            if (validResult && validResult.length > 0) {
                validResult = result.filter(function (r) { return r.ResourceGroup === azureCredentials.resourceGroup; });
                if (validResult && validResult.length > 0) {
                    validResult = result.filter(function (r) { return r.AccountName === azureCredentials.accountName; });
                    if (validResult && validResult.length > 0) {
                        resultDescription = 'OK';
                    }
                    else {
                        resultDescription = 'SERVER_ERROR_NLU_ACCOUNTNAME_NOT_FOUND';
                    }
                }
                else {
                    resultDescription = 'SERVER_ERROR_NLU_RESOURCEGROUP_NOT_FOUND';
                }
            }
            else {
                resultDescription = 'SERVER_ERROR_NLU_SUBSCRIPTION_NOT_FOUND';
            }
            if (validResult && validResult.length > 0)
                isValid = true;
            if (res.headersSent)
                return;
            res.json({ status: true, description: (isValid ? 'OK' : 'VALIDATE_AZURE_ACCOUNTS_FAILED'), data: { result: isValid, description: resultDescription } });
        })
            .catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'ERROR_COGNITIVE_SERVICE_VALIDATE_AZURE_CREDENTIALS');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Error al ejecutar solicitud');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            logger.error('[NLUService::validate_azure_credentials] getting errorMsg:', errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: clientMsg });
        });
    });
    app.post('/api/nlu/get_nlus/', function (req, res, next) {
        logger.info('[NLUService::get_nlus] Received ' + req.method + ' method on \'' + req.path + '\'');
        logger.info('[NLUService::get_nlus] getting search_criteria:', req.params.search_criteria, 'search_expression:', req.params.search_expression);
        var vcc = res.locals.vcc;
        var azure = res.locals.azure;
        logger.info('[NLUService::get_nlus] getting vcc:', vcc, 'azure:', azure);
        var search_criteria = req.body.search_criteria || '';
        var search_expression = req.body.search_expression || '';
        var isSystemApp = req.body.isSystemApp || false;
        promises_shared_1.DummyPromise()
            .then(function (result) {
            if (isSystemApp) {
                return cognitive_analysis_implementation_1.GetCognitiveService(null, 'RASA');
            }
            else {
                return cognitive_analysis_implementation_1.GetCognitiveService(vcc, null);
            }
        }).then(function (result) {
            cognitiveService = result;
            return cognitiveService.GetAppsList(azure.subscriptionRegion, azure.subscriptionKey);
        })
            .then(function (result) {
            var arrayId = [], stringId = '';
            if (result && result.length) {
                arrayId = result.map(function (a) { return a.id; });
                logger.info('[NLUService::get_nlus] getting arrayId:', arrayId.length);
                stringId = arrayId.join('\',\'');
                logger.info('[NLUService::get_nlus] getting stringId:', stringId);
            }
            return mssql_service_1.MSSql.GetNLUs(vcc, search_criteria, search_expression, stringId, isSystemApp);
        })
            .then(function (nlus) {
            logger.info('[NLUService::get_nlus] getting nlus:', nlus.length);
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'OK', data: nlus });
        })
            .catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'ERROR_COGNITIVE_SERVICE_GET_NLUS');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Error al ejecutar solicitud');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            logger.error('[NLUService::get_nlus] getting errorMsg:', errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: clientMsg });
        });
    });
    app.get('/api/nlu/get_cultures', function (req, res, next) {
        logger.info('Se recibió ' + req.method + ' para \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        logger.info('[NLUService::/api/nlu/get_cultures] getting vcc:', vcc);
        var azure = res.locals.azure;
        logger.info('[NLUService::/api/nlu/get_cultures] getting azure:', azure);
        promises_shared_1.DummyPromise()
            .then(function (result) {
            return cognitive_analysis_implementation_1.GetCognitiveService(vcc, null);
        }).then(function (result) {
            cognitiveService = result;
            return cognitiveService.GetCultures(azure.subscriptionRegion, azure.subscriptionKey);
        })
            .then(function (cultures) {
            logger.info('[NLUService::/api/nlu/get_cultures] getting cultures:', cultures, 'after cognitiveService.GetCultures');
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'OK', data: cultures });
        })
            .catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Error al ejecutar solicitud');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            logger.error('[NLUService::/api/nlu/get_cultures] getting errorMsg:', errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.get('/api/nlu/get_nlu/:nlu_id', function (req, res, next) {
        logger.info('[NLUService::get_nlu] Received ' + req.method + ' method on \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var azure = res.locals.azure;
        var nluId = req.params.nlu_id;
        var nlu = null;
        promises_shared_1.DummyPromise()
            .then(function (result) {
            return mssql_service_1.MSSql.GetNLUByAppId(vcc, nluId);
        })
            .then(function (resultNLU) {
            logger.info('[NLUService::get_nlu] getting resultNLU:', resultNLU, 'after MSSql.GetNLU');
            nlu = resultNLU;
            if (nlu.isSystemApp) {
                return cognitive_analysis_implementation_1.GetCognitiveService(null, 'RASA');
            }
            else {
                return cognitive_analysis_implementation_1.GetCognitiveService(vcc, null);
            }
        })
            .then(function (result) {
            cognitiveService = result;
            if (!nluId)
                throw 'MISSING_NLU_INPUT_PARAMETER';
            return cognitiveService.GetAppVersion(azure.subscriptionRegion, azure.subscriptionKey, nlu.appId);
        })
            .then(function (result) {
            logger.info('[NLUService::get_nlu] getting result:', result, 'after cognitiveService.GetAppVersion');
            nlu.version = result.version;
            nlu.createdDateTime = result.createdDateTime;
            nlu.lastModifiedDateTime = result.lastModifiedDateTime;
            nlu.lastTrainedDateTime = result.lastTrainedDateTime;
            nlu.lastPublishedDateTime = result.lastPublishedDateTime;
            nlu.endpointUrl = result.endpointUrl;
            nlu.assignedEndpointKey = result.assignedEndpointKey;
            nlu.externalApiKeys = result.externalApiKeys;
            nlu.intentsCount = result.intentsCount;
            nlu.entitiesCount = result.entitiesCount;
            nlu.endpointHitsCount = result.endpointHitsCount;
            nlu.trainingStatus = result.trainingStatus;
            return nlu;
        })
            .then(function (nluModel) {
            logger.info('[NLUService::get_nlu] getting nluModel:', nluModel);
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'OK', data: nluModel });
        })
            .catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Error al ejecutar solicitud');
            logger.error('[NLUService::get_nlu] getting errorMsg:', errorMsg, 'nluId:', nluId);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/nlu/get_unique_nlu_by_intents', function (req, res, nex) {
        var vcc = res.locals.vcc;
        var templateId = (req.body && req.body.templateId ? req.body.templateId : null);
        var intents = (req.body && req.body.intents ? req.body.intents : {});
        promises_shared_1.DummyPromise()
            .then(function (result) {
            var intentsString = '';
            if (intents && intents.length) {
                intentsString = intents.join(',');
            }
            return mssql_service_1.MSSql.GetUniqueNLUByIntents(vcc, templateId, intentsString);
        }).then(function (result) {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'OK', data: result });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Error al ejecutar solicitud');
            logger.error('[NLUService::/api/nlu/get_unique_nlu_by_intents] Getting errorMsg:', errorMsg, 'intents:', intents);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/nlu/create_nlu', function (req, res, next) {
        req.setTimeout(300000, function () { });
        var vcc = res.locals.vcc;
        var azure = res.locals.azure;
        var userId = res.locals.userId;
        var nowUtc = moment().utc().toDate();
        var newNLU = (req.body || {});
        var saveNewNLUResult = null;
        newNLU.vcc = vcc;
        newNLU.createdDate = nowUtc;
        newNLU.createdByUserId = userId;
        newNLU.lastModifiedDate = nowUtc;
        newNLU.lastModifiedByUserId = userId;
        logger.info('[NLUService::create_nlu] getting newNLU:', newNLU);
        promises_shared_1.DummyPromise()
            .then(function (result) {
            return cognitive_analysis_implementation_1.GetCognitiveService(vcc, null);
        }).then(function (result) {
            cognitiveService = result;
            switch (cognitiveService.GetCognitiveEngineName()) {
                case 'LUIS': return mssql_service_1.MSSql.GetNLUByName(null, newNLU.name);
                case 'RASA': return mssql_service_1.MSSql.GetNLUByName(vcc, newNLU.name);
                default: throw 'SERVER_ERROR_UNKNOWN_COGNITIVE_ENGINE';
            }
        })
            .then(function (resultNLU) {
            logger.info('[NLUService::create_nlu] getting resultNLU:', resultNLU, 'after MSSql.GetNLUByName');
            if (resultNLU && resultNLU.length > 0)
                throw 'MODEL_NLU_ALREADY_EXIST \'' + newNLU.name + '\'';
            return cognitiveService.AddApp(azure && azure.subscriptionRegion ? azure.subscriptionRegion : null, azure && azure.subscriptionKey ? azure.subscriptionKey : null, newNLU);
        })
            .then(function (result) {
            logger.info('[NLUService::create_nlu] getting result:', result, 'after cognitiveService.AddApp');
            newNLU.appId = result;
            return SaveNLU(vcc, newNLU);
        })
            .then(function (resultSaveNLU) {
            logger.info('[NLUService::create_nlu] getting resultSaveNLU:', resultSaveNLU, 'after SaveNLU');
            if (!resultSaveNLU)
                throw 'ERROR_SAVING_MODEL_NLU';
            saveNewNLUResult = resultSaveNLU;
            if (newNLU.importNLU) {
                return ImportNLUIntents(vcc, newNLU, cognitiveService);
            }
            else {
                return saveNewNLUResult;
            }
        })
            .then(function (resultSaveNLU) {
            if (res.headersSent)
                return;
            res.json({ status: true, description: (saveNewNLUResult ? 'OK' : 'MODEL_SAVE_FAILED'), data: saveNewNLUResult });
        })
            .catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Error al ejecutar solicitud');
            if (err.error && err.error.error && err.error.error.message) {
                var message = '';
                var message2 = message.concat('An application with the same name (', newNLU.name, ') already exists.');
                if (err.error.error.message === message2) {
                    clientMsg = 'MODEL_NLU_INVALID_NAME';
                    errorMsg = 'MODEL_NLU_INVALID_NAME';
                }
            }
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            logger.error('[NLUService::create_nlu] getting errorMsg:', errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/nlu/save_nlu', function (req, res, next) {
        logger.info('Se recibió ' + req.method + ' para \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        logger.info('[NLUService::/api/nlu/save_nlu] getting vcc:', vcc);
        var userId = res.locals.userId;
        logger.info('[NLUService::/api/nlu/save_nlu] getting userId:', userId);
        var nowUtc = moment().utc().toDate();
        logger.info('[NLUService::/api/nlu/save_nlu] getting nowUtc:', nowUtc);
        var newNLU = (req.body || {});
        newNLU.createdDate = null;
        newNLU.createdByUserId = null;
        newNLU.lastModifiedDate = nowUtc;
        newNLU.lastModifiedByUserId = userId;
        logger.info('[NLUService::/api/nlu/save_nlu] getting newNLU:', newNLU);
        promises_shared_1.DummyPromise()
            .then(function (result) {
            return SaveNLU(vcc, newNLU);
        })
            .then(function (resultSaveNLU) {
            logger.info('[NLUService::/api/nlu/save_nlu] getting resultSaveNLU:', resultSaveNLU);
            if (!resultSaveNLU)
                throw 'ERROR_SAVING_MODEL_NLU';
            return Promise.resolve(true);
        })
            .then(function (resultSaveNLU) {
            if (res.headersSent)
                return;
            res.json({ status: resultSaveNLU, description: (resultSaveNLU ? 'OK' : 'MODEL_SAVE_FAILED'), data: resultSaveNLU });
        })
            .catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Error al ejecutar solicitud');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            logger.error('[NLUService::/api/nlu/save_nlu] getting errorMsg:', errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/nlu/delete_nlu', function (req, res, next) {
        logger.info('[NLUService::delete_nlu] Received ' + req.method + ' method on \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        logger.info('[NLUService::delete_nlu] getting vcc:', vcc);
        var azure = res.locals.azure;
        logger.info('[NLUService::delete_nlu] getting azure:', azure);
        var nlu = req.body.nlu;
        logger.info('[NLUService::delete_nlu] getting nlu:', nlu);
        var nluId = nlu.name;
        logger.info('[NLUService::delete_nlu] getting nluId:', nluId);
        promises_shared_1.DummyPromise()
            .then(function (result) {
            return cognitive_analysis_implementation_1.GetCognitiveService(vcc, null);
        }).then(function (result) {
            cognitiveService = result;
            if (!nlu)
                throw 'MISSING_NLU_INPUT_PARAMETER';
            nlu.vcc = vcc;
            return cognitiveService.DeleteApp(azure.subscriptionRegion, azure.subscriptionKey, nlu);
        }).then(function (result) {
            return mssql_service_1.MSSql.DeleteNLU(vcc, nluId);
        }).then(function (resultDeleteNLU) {
            logger.info('[NLUService::delete_nlu] getting resultDeleteNLU:', resultDeleteNLU);
            if (!resultDeleteNLU)
                throw 'ERROR_DELETING_MODEL_NLU';
            return Promise.resolve(true);
        })
            .then(function (resultDeleteNLU) {
            if (res.headersSent)
                return;
            res.json({ status: resultDeleteNLU, description: (resultDeleteNLU ? 'OK' : 'MODEL_DELETE_FAILED'), data: resultDeleteNLU });
        })
            .catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Error al ejecutar solicitud');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            logger.error('[NLUService::delete_nlu] getting errorMsg:', errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/nlu/delete_nlus', function (req, res, next) {
        logger.info('Se recibió ' + req.method + ' para \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        logger.info('[NLUService::/api/nlu/delete_nlus] getting vcc:', vcc);
        var nlus = req.body.nlus || [];
        logger.info('[NLUService::/api/nlu/delete_nlus] getting nlus:', nlus);
        promises_shared_1.DummyPromise()
            .then(function (result) {
            if (!nlus)
                throw 'MISSING_MODELS_NLU_INPUT_PARAMETER';
            var promiseArray = [];
            nlus.forEach(function (nlu) {
                promiseArray.push(mssql_service_1.MSSql.DeleteNLU(vcc, nlu.name));
            });
            return Promise.all(promiseArray);
        })
            .then(function (resultDeleteNLUs) {
            logger.info('[NLUService::/api/nlu/delete_nlus] getting resultDeleteNLUs:', resultDeleteNLUs);
            var result;
            if (resultDeleteNLUs.filter(function (result) { return (result !== true); }).length > 0) {
                result = false;
                logger.error('[NLUService::/api/nlu/delete_nlus] could not delete all requested items');
            }
            else {
                result = true;
                logger.info('[NLUService::/api/nlu/delete_nlus] models deleted successfully');
            }
            return result;
        })
            .then(function (resultDeleteNLUs) {
            if (res.headersSent)
                return;
            res.json({ status: resultDeleteNLUs, description: (resultDeleteNLUs ? 'OK' : 'MODELS_DELETE_FAILED'), data: resultDeleteNLUs });
        })
            .catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Error al ejecutar solicitud');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            logger.error('[NLUService::/api/nlu/delete_nlus] getting errorMsg:', errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/nlu/get_subscriptionkey', function (req, res, next) {
        logger.info('[NLUService::get_subscriptionkey] Received ' + req.method + ' method on \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var azureSubscriptionKey = req.body;
        logger.info('[NLUService::get_subscriptionkey] getting vcc:', vcc);
        promises_shared_1.DummyPromise()
            .then(function (result) {
            if (azureSubscriptionKey.subscriptionName !== null)
                return mssql_service_1.MSSql.GetAzureSubscriptionKeyByName(vcc, azureSubscriptionKey.subscriptionName);
            else if (azureSubscriptionKey.subscriptionKey !== null)
                return mssql_service_1.MSSql.GetAzureSubscriptionKeyByKey(vcc, azureSubscriptionKey.subscriptionKey);
            else
                throw 'SERVER_ERROR_MISSING_PARAMETERS_FOR_SEARCH';
        })
            .then(function (result) {
            logger.info('[NLUService::get_subscriptionkey] getting keys:', result, 'after MSSql.GetSubscriptionKeys');
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'OK', data: result });
        })
            .catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Error al ejecutar solicitud');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            logger.error('[NLUService::get_subscriptionkeys] getting errorMsg:', errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/nlu/get_subscriptionkeys', function (req, res, next) {
        logger.info('[NLUService::get_subscriptionkeys] Received ' + req.method + ' method on \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        logger.info('[NLUService::get_subscriptionkeys] getting vcc:', vcc);
        promises_shared_1.DummyPromise()
            .then(function (result) {
            return mssql_service_1.MSSql.GetAzureSubscriptionKeys(vcc);
        })
            .then(function (result) {
            logger.info('[NLUService::get_subscriptionkeys] getting keys:', result.length, 'after MSSql.GetSubscriptionKeys');
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'OK', data: result });
        })
            .catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Error al ejecutar solicitud');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            logger.error('[NLUService::get_subscriptionkeys] getting errorMsg:', errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/nlu/upload_file/:batch_type_name?/:upload_dest?', function (req, res, next) {
        logger.info('[NLUService::upload_file] Received ' + req.method + ' method on \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var userId = res.locals.userId;
        var cognitiveEngineSource = req.params.batch_type_name ? req.params.batch_type_name : null;
        var nowUtc = moment().utc().toDate();
        var uploadToFolder = req.params.upload_dest || '';
        var nluFileRedisKey = 'NLU_FILE:' + userId;
        var nluFileContent = null;
        var resultUpload = false;
        var tempFolderName = (uploadToFolder ? uploadToFolder : (config_1.NLU_TEMP_PREFIX + '-' + vcc + '-' + userId).toLowerCase() + '-' + nowUtc.valueOf());
        var tempZipFileName = tempFolderName + '.md';
        var tempUploadPath = (uploadToFolder ? uploadToFolder : config_1.NLU_FOLDER_TEMP_PATH);
        promises_shared_1.DummyPromise()
            .then(function (result) {
            return cognitive_analysis_implementation_1.GetCognitiveService(vcc, null);
        }).then(function (result) {
            cognitiveService = result;
            if (!nluFileRedisKey)
                throw 'SERVER_ERROR_NLU_TARGET_NOT_FOUND';
            logger.info('[NLUService::upload_file] Getting tempFolderName:', tempFolderName, 'with tempZipFileName:', tempZipFileName, 'with tempUploadPath:', tempUploadPath);
            return file_manager_1.ExistInFileSystem(tempUploadPath);
        }).then(function (result) {
            if (result === false) {
                return file_manager_1.CreateFolder(tempUploadPath);
            }
            else {
                return true;
            }
        }).then(function (result) {
            if (result) {
                return file_manager_1.UploadFileToServer(tempZipFileName, tempUploadPath, req, res, '(md|json)');
            }
            else {
                return false;
            }
        }).then(function (result) {
            resultUpload = result;
            logger.info('[NLUServiceService::upload_file] Getting resultUpload:', resultUpload, 'after UploadFileToServer');
            if (!resultUpload)
                throw 'SERVER_ERROR_COULD_NOT_UPLOAD_FILE';
            var fileName = tempUploadPath + "/" + tempZipFileName;
            return file_manager_1.ReadFileContent(fileName, 'utf-8');
        }).then(function (result) {
            if (!result)
                throw 'SERVER_ERROR_CANNOT_READ_FILE';
            var breakLineRegex = new RegExp("(\r\n|\r)", 'gm');
            nluFileContent = result.replace(breakLineRegex, '\n');
            if (nluFileContent && nluFileContent.length > 0) {
                return redis_service_1.Redis.SaveFileNLU(vcc, nluFileRedisKey, nluFileContent);
            }
            else
                throw 'SERVER_ERROR_UPLOADED_FILE_EMPTY';
        }).then(function (result) {
            if (!result)
                throw 'SERVER_ERROR_COULDNOT_SAVE_FILE_CONTENT';
            logger.info('[NLUServiceService::upload_file] Getting result:', result, 'after ReadFileContent');
            if (result === true) {
                var tempCSVFile = tempUploadPath + "/" + tempZipFileName;
                logger.info('[NLUServiceService::upload_file] Getting tempCSVFile');
                return file_manager_1.DeleteFile(tempCSVFile);
            }
            else {
                return Promise.resolve(false);
            }
        }).then(function (result) {
            if (result === true) {
                return redis_service_1.Redis.GetFileNLU(vcc, (nluFileRedisKey));
            }
            else {
                return Promise.resolve(null);
            }
        }).then(function (result) {
            var intents = [];
            var utterances = [];
            switch (cognitiveEngineSource) {
                case 'RASA':
                    intents = cognitiveService.GetAllIntentsFromRASAContent(result.data);
                    utterances = cognitiveService.GetAllUtterancesFromRASAContent(result.data);
                    break;
            }
            if (intents.length === 0)
                throw 'SERVER_ERROR_INTENTS_NOT_FOUND';
            intents.map(function (i) {
                if (i.name.toLowerCase() === 'none')
                    throw 'SERVER_ERROR_INTENT_ID_INVALID';
                if (validate_shared_1.IsStringAndEmpty(i.name))
                    throw 'SERVER_ERROR_INTENT_ID_MISSING';
                if (!validate_shared_1.IsStringAndValiadId(i.name))
                    throw 'SERVER_ERROR_INTENT_ID_INVALID';
            });
            if (res.headersSent)
                return;
            var data = {
                tempFolderName: tempFolderName,
                content: result.data,
                intents: intents.length,
                utterances: utterances.length
            };
            res.json({ status: resultUpload, description: (resultUpload ? 'FILE_STORED_SUCCESSFULLY' : 'FILE_UPLOAD_FAILED'), data: data });
        }).catch(function (err) {
            var processMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Error al ejecutar solicitud');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            logger.error('[NLUService::upload_file] Getting errorMsg:', errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: processMsg, error: errorMsg });
        });
    });
    app.post('/api/nlu/delete_file_temporal/', function (req, res, next) {
        logger.info('[NLUService::delete_file_temporal] Received ' + req.method + ' method on \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var userId = res.locals.userId;
        var nluFileRedisKey = 'NLU_FILE:' + userId;
        logger.info('[NLUService::delete_file_temporal] Getting vcc:userId : ', vcc, ':', userId);
        promises_shared_1.DummyPromise()
            .then(function (result) {
            return redis_service_1.Redis.GetFileNLU(vcc, nluFileRedisKey);
        }).then(function (result) {
            if (result && result.data.length > 0) {
                return redis_service_1.Redis.DeleteFileNLU(vcc, nluFileRedisKey);
            }
            else {
                return Promise.resolve(true);
            }
        }).then(function (result) {
            if (res.headersSent)
                return;
            res.json({ status: result, description: (result ? 'CLEAR SUCCESS' : 'CLEAR ERROR '), data: result });
        }).catch(function (err) {
            var processMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Error al ejecutar solicitud');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            logger.error('[NLUService::delete_file_temporal] Getting errorMsg:', errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: processMsg, error: errorMsg });
        });
    });
    app.post('/api/nlu/export_nlu', function (req, res, next) {
        logger.info('[NLUService::export_nlu] Received ' + req.method + ' method on \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        logger.info('[NLUService::delete_nlu] getting vcc:', vcc);
        var nlu = req.body.nlu;
        nlu.vcc = vcc;
        logger.info('[NLUService::delete_nlu] getting nlu:', nlu);
        promises_shared_1.DummyPromise()
            .then(function (result) {
            return cognitive_analysis_implementation_1.GetCognitiveService(vcc, null);
        }).then(function (result) {
            cognitiveService = result;
            res.setHeader('Content-type', 'application/octet-stream');
            res.setHeader('Content-disposition', 'attachment; filename=');
            return cognitiveService.ExportModel(nlu, res);
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Error al ejecutar solicitud');
            logger.error('[NLUService::export_nlu] Getting errorMsg:', errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
}
exports.NLUService = NLUService;
function GetNLU(vcc, name) {
    logger.info('[NLUService::GetNLU] getting vcc:', vcc, 'name:', name);
    var cacheUpdateNeeded = false;
    return Promise.resolve()
        .then(function () {
        return mssql_service_1.MSSql.GetNLU(vcc, name);
    })
        .catch(function (err) {
        var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Error al ejecutar solicitud');
        logger.error('[NLUService::GetNLU] getting errorMsg:', errorMsg);
    });
}
exports.GetNLU = GetNLU;
function SaveNLU(vcc, nlu) {
    logger.info('[NLUService::SaveNLU] getting vcc:', vcc, 'nlu:', nlu);
    var promiseArray = [];
    promiseArray.push(mssql_service_1.MSSql.SaveNLU(vcc, nlu.name, nlu.culture, nlu.description, nlu.appId, nlu.subscriptionKey, nlu.isSystemApp, nlu.needTrain, nlu.isPublished, nlu.lastPublishedDateTime, nlu.isTrained, nlu.lastTrainedDateTime, nlu.createdDate, nlu.createdByUserId, nlu.lastModifiedDate, nlu.lastModifiedByUserId));
    return Promise.all(promiseArray)
        .then(function (results) {
        logger.info('[NLUService::SaveNLU] getting results:', results, 'after Promise.all MSSql.SaveNLU');
        if (results.filter(function (result) { return (result !== true); }).length > 0) {
            return false;
        }
        else {
            return true;
        }
    });
}
exports.SaveNLU = SaveNLU;
function ImportNLUIntents(vcc, nlu, cognitiveService) {
    logger.info('[NLUService::ImportNLUIntents] getting vcc:', vcc, 'nlu:', nlu.appId);
    var nluContent = null;
    var nluFileRedisKey = 'NLU_FILE:' + nlu.createdByUserId;
    return promises_shared_1.DummyPromise().then(function (result) {
        return redis_service_1.Redis.GetFileNLU(nlu.vcc, (nluFileRedisKey));
    }).then(function (result) {
        if (!result)
            throw 'SERVER_ERROR_NLU_REDIS_DATA_NOT_FOUND';
        var arrIntents = [];
        nluContent = result.data;
        switch (nlu.importNLUFrom) {
            case 'RASA': arrIntents = cognitiveService.GetAllIntentsFromRASAContent(nluContent);
        }
        var promiseArray = [];
        arrIntents.map(function (ai) {
            if (ai.name.toLowerCase() === 'none')
                throw 'SERVER_ERROR_INTENT_ID_INVALID';
            var newIntent = {};
            newIntent.appId = nlu.appId;
            newIntent.name = ai.name;
            newIntent.intentId = ai.name;
            newIntent.createdDate = moment().utc().toDate();
            newIntent.createdByUserId = nlu.createdByUserId;
            newIntent.color = utilities_1.getRandomHexColor();
            promiseArray.push(intent_service_1.SaveIntent(vcc, newIntent));
        });
        return Promise.all(promiseArray);
    }).then(function (result) {
        return redis_service_1.Redis.DeleteFileNLU(nlu.vcc, (nluFileRedisKey));
    }).then(function (result) {
        return Promise.resolve(true);
    }).catch(function (err) {
        var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Error al ejecutar solicitud');
        logger.error('[NLUService::GetNLU] getting errorMsg:', errorMsg);
        return Promise.reject(errorMsg);
    });
}
